/* Copyright 2010 Markus Peloquin <markus@cs.wisc.edu>,
 * Lena Olson <lena@cs.wisc.edu>
 *
 * You can't do anything with this but read and execute it privately.  If you
 * want to do more, ask us if we'd be willing to make a more permissive
 * license. */

#ifndef HRTIMER_H
#define HRTIMER_H

// gethrtime already defined
#include <sys/time.h>
#include <cassert>
#include <cstdio>
#include <cstring>
#include <ctime>
#include <tr1/cstdint>

#if defined(__sparc)

#elif defined(__i386) || defined(__tune_i686__) || defined(__x86_64__) || defined(__x86_64) || defined(__amd64)
 
typedef long long int hrtime_t;

inline hrtime_t rdtsc() {
  unsigned long int lo, hi;
  __asm__ __volatile__ ("rdtsc" : "=a" (lo), "=d" (hi));
  return (hrtime_t) ((hi << 32) | lo);
}

#else // not sparc or x86
#error "HRTimer not supported on this architecture/OS"
#endif

extern double m_cpu_mhz;

inline void init_timer(){
#if defined(__linux) || defined(__linux__) || defined(linux)
    FILE* cpuinfo;
    char str[100];
    cpuinfo = fopen("/proc/cpuinfo","r");
    while(fgets(str,100,cpuinfo) != NULL){
      char cmp_str[8];
      strncpy(cmp_str, str, 7);
      cmp_str[7] = '\0';
      if(strcmp(cmp_str, "cpu MHz") == 0){
        double cpu_mhz;
        sscanf(str, "cpu MHz : %lf", &cpu_mhz);
        m_cpu_mhz = cpu_mhz;
        break;
      }
    }
    
    fclose( cpuinfo );
#endif
}

inline double get_time_ns() {
#if defined(__sparc)
    return gethrtime();
#elif defined(__i386) || defined(__tune_i686__) || defined(__x86_64__) || defined(__x86_64) || defined(__amd64)
    return rdtsc()*1.0e3/m_cpu_mhz;
#endif
}

static inline uint64_t clock_monotonic()
{
	struct timespec ts;
	/* the only possible error is that the clock is not available */
	if (clock_gettime(CLOCK_MONOTONIC, &ts) == -1) assert(0);
	return (uint64_t)ts.tv_sec * 1000000000ULL + ts.tv_nsec;
}

#endif
