/* Copyright 2010 Markus Peloquin <markus@cs.wisc.edu>,
 * Lena Olson <lena@cs.wisc.edu>
 *
 * You can't do anything with this but read and execute it privately.  If you
 * want to do more, ask us if we'd be willing to make a more permissive
 * license. */

__kernel void bucketfill(
  /* params */
  __global int *array, //read-write
  int dim_x,
  int dim_y,
  int old_val,
  int new_val,

  unsigned max_tid,

  /* args */
  __constant int *arg_x,
  __constant int *arg_y,

  /* results */
  __global int *result,
  __global unsigned *spawn,

  /* spawn args */
  __global int *spawn_x,
  __global int *spawn_y
)
{
  unsigned tid = get_global_id(0);
  if (tid > max_tid) return;

  int x = arg_x[tid];
  int y = arg_y[tid];


  if (x > dim_x || x < 0 || y < 0 || y > dim_y){
    spawn[tid] = false;
    return;
  }

  if (array[x*dim_x+y] != old_val || array[x*dim_x+y] == new_val){
    spawn[tid] = false;
    return;
  }

  array[x*dim_x+y] = new_val;

  spawn_x[tid*4] = x-1;
  spawn_y[tid*4] = y;

  spawn_x[tid*4 + 1] = x+1;
  spawn_y[tid*4 + 1] = y;

  spawn_x[tid*4 + 2] = x;
  spawn_y[tid*4 + 2] = y-1;

  spawn_x[tid*4 + 3] = x;
  spawn_y[tid*4 + 3] = y+1;

  spawn[tid] = true;
}

/* vim: set filetype=c : */
