/* Copyright 2010 Markus Peloquin <markus@cs.wisc.edu>,
 * Lena Olson <lena@cs.wisc.edu>
 *
 * You can't do anything with this but read and execute it privately.  If you
 * want to do more, ask us if we'd be willing to make a more permissive
 * license. */

#include <cassert>
#include <iostream>

#include "HRTimer.h"
#include "openclk.hpp"

namespace {

  class Bucketfill : public Alg_spec<cl_int> {
    public:
      Bucketfill(int * array, int dim_x, int dim_y, int x, int y, int old_val, int new_val)
      {
        add_param(array, dim_x*dim_y*sizeof(cl_int), Argument_buffer::read_write);
        add_param(static_cast<cl_int>(dim_x));
        add_param(static_cast<cl_int>(dim_y));
        add_param(static_cast<cl_int>(old_val));
        add_param(static_cast<cl_int>(new_val));
        add_arg(static_cast<cl_int>(x));
        add_arg(static_cast<cl_int>(y));
      }

    protected:
      virtual std::string name() const
      { return "bucketfill"; }

      virtual void reduce(const cl_int *child_results, size_t len,
                          cl_int *result)
      {
      }

      virtual unsigned sub_tasks() const
      { return 4; }
  };
  
  // wooo!
  void floodFill(Openclk &openclk, int * array, int dim_x, int dim_y, int x,
                int y, int old_val, int new_val)
  {
    Bucketfill bucketfill(array, dim_x, dim_y, x, y, old_val, new_val);
    bucketfill.run(openclk);
  }

} // end anon namespace

void initField(int * array, int dim_x, int dim_y){
  for (int i=0; i < dim_x; i++){
    for (int j=0; j < dim_y; j++){  
      array[i*dim_x+j] = 0;
    }
  }

  for (int i=0; i < dim_x; i++){
    array[i*dim_x+2] = 5;
    array[i*dim_x + dim_y - 3] = 5;
    array[4*dim_x+i] = 5;
    array[dim_x*(dim_x - 2)+i] = 5;
  }
}

void printField(int * array, int dim_x, int dim_y){
  for (int i=0; i < dim_x; i++){
    for (int j=0; j < dim_y; j++){  
      std::cout << array[dim_x*i+j];
    }
    std::cout << "\n";
  }
}

int main(int argc, char ** argv){

  int x_dim = 100;
  int y_dim = 100;

  if (argc == 3){
    x_dim = atoi(argv[1]);
    y_dim = atoi(argv[2]);
  }

  int * array = new int[x_dim*y_dim];
  initField(array, x_dim, y_dim);
  printField(array, x_dim, y_dim);

  const char *source_files[] = {
    "bucketfill.cl",
  };
  size_t count = sizeof(source_files) / sizeof(*source_files);

     // initialize 
  Openclk openclk(source_files, source_files + count);

  double start, end;
  init_timer();
  start = get_time_ns();
  floodFill(openclk, array, x_dim, y_dim, x_dim/2, y_dim/2, 0, 9);
  end = get_time_ns();
  std::cout << "final:\n"; 
  printField(array, x_dim, y_dim);
  std::cout << "Time: " << (end - start) / 1000000000.0 << " s\n";
  
  return 0; 
}

