/* Copyright 2010 Markus Peloquin <markus@cs.wisc.edu>,
 * Lena Olson <lena@cs.wisc.edu>
 *
 * You can't do anything with this but read and execute it privately.  If you
 * want to do more, ask us if we'd be willing to make a more permissive
 * license. */

__kernel void determinant(
	/* params */
	unsigned dim,

	unsigned max_tid,

	/* args */
	__constant float *arg_b,
	__constant int *arg_m,

	/* results  */
	__global int *result,
	__global unsigned *spawn,

	/* spawn args */
	__global float *spawn_b,
	__global int *spawn_m,
)
{
	unsigned tid = get_global_id(0);
	if (tid > max_tid) return;

	float *b = arg_b[dim*dim*tid]; 
	int *m = arg_m[tid];
	
	if (tid > m){
		spawn[tid] = false;
		result[tid] = 0;
	}
	if (m == 2){ // c-ase of base
		spawn[tid] = false;
		result[tid] = b[0]*b[dim+1] - b[1]*b[dim];
	}

	float sum = 0;
	float *c = new float[dim * dim];
	for (int p = 0; p < dim; p++) {
		if (p < m) {
			int h = 0;
			int k = 0;
			for (int i = 1; i < m; i++) {
				for (int j = 0; j < m; j++) {
					if (j == p)
						continue;
					c[h*dim + k] = b[i*dim + j];
					k++;
					if (k == m-1){
						h++;
						k = 0;
					}
				}
			}
		}

		sum += b[p] * pow(-1, p) * determinant(dim, c, m-1);
		spawn_m[tid*dim + p] = m-1;
	}
	spawn[tid] = true;
	return sum;
}

/* vim: set filetype=c : */
