/* Copyright 2010 Markus Peloquin <markus@cs.wisc.edu>,
 * Lena Olson <lena@cs.wisc.edu>
 *
 * You can't do anything with this but read and execute it privately.  If you
 * want to do more, ask us if we'd be willing to make a more permissive
 * license. */

#include <iostream>
#include <cstdlib>
#include <math.h>
#include "HRTimer.h"

#include "openclk.hpp"

extern char *__progname;

namespace {

  class Determinant : public Alg_spec<cl_float> {
    public:
      Determinant(int dim, float * b)
      {
	max_spawn = dim;
        add_param(array, len * sizeof(cl_uint), Argument_buffer::read_write);
      }

    protected:
      virtual std::string name() const
      { return "determinant"; }

      virtual void reduce(const cl_uint *child_results, size_t len,
                          cl_uint *result)
      {
	return;
      }

      virtual unsigned sub_tasks() const
      { return max_spawn; }
      
      int max_spawn;

  };

// wooo!
  unsigned
     determinant(Openclk &openclk, int dim, const float *b)
  {
    Determinant determinant(dim, b);
    return determinant.run(openclk);
  }

} // end anon namespace

void
fill_random(float *arr, size_t num)
{
	while (num--)
		*arr++ = rand() % 100;
}

int
main(int argc, char **argv)
{
  // dimensions
  unsigned dim = 5;
  if (argc > 1){
    dim = atoi(argv[1]);
  }

  const char *source_files[] = {
    "determinant.cl",
  };
  size_t count = sizeof(source_files) / sizeof(*source_files);

  // initialize 
  Openclk openclk(source_files, source_files + count);


  srand(0);
  float *A = new float[dim * dim];

  fill_random(A, dim*dim);

  init_timer();
  double begin = get_time_ns();
  float result = determinant(openclk, dim, A);
  double end = get_time_ns();

  delete[] A;

  std::cout << "result: " << result << "\n";
  std::cout << "time: " << (end - begin) / 1000000000.0 << " s\n";

  return 0;
}
