/* Copyright 2010 Markus Peloquin <markus@cs.wisc.edu>,
 * Lena Olson <lena@cs.wisc.edu>
 *
 * You can't do anything with this but read and execute it privately.  If you
 * want to do more, ask us if we'd be willing to make a more permissive
 * license. */

#include <iostream>
#include <cstdlib>
#include <math.h>
#include "HRTimer.h"

float determinant(int dim, const float *b, int m, int tid)
{
	if (tid > m)
		return 0;
	if (m == 2) // c-ase of base
		return b[0]*b[dim+1] - b[1]*b[dim];

	float sum = 0;
	float *c = new float[dim * dim];
	for (int p = 0; p < dim; p++) {
		if (p < m) {
			int h = 0;
			int k = 0;
			for (int i = 1; i < m; i++) {
				for (int j = 0; j < m; j++) {
					if (j == p)
						continue;
					c[h*dim + k] = b[i*dim + j];
					k++;
					if (k == m-1){
						h++;
						k = 0;
					}
				}
			}
		}

		sum += b[p] * pow(-1, p) * determinant(dim, c, m-1, p);
	}
	delete[] c;
	return sum;
}

float determinant(int dim, const float *b)
{
	return determinant(dim, b, dim, 0);
}

void
fill_random(float *arr, size_t num)
{
	while (num--)
		*arr++ = rand() % 100;
}

int
main(int argc, char **argv)
{
  // dimensions
  unsigned dim = 5;
  if (argc > 1){
    dim = atoi(argv[1]);
  }


  srand(0);
  float *A = new float[dim * dim];

  fill_random(A, dim*dim);

  init_timer();
  double begin = get_time_ns();
  float result = determinant(dim, A);
  double end = get_time_ns();

  delete[] A;

  std::cout << "result: " << result << "\n";
  std::cout << "time: " << (end - begin) / 1000000000.0 << " s\n";

  return 0;
}

