/* Copyright 2010 Markus Peloquin <markus@cs.wisc.edu>,
 * Lena Olson <lena@cs.wisc.edu>
 *
 * You can't do anything with this but read and execute it privately.  If you
 * want to do more, ask us if we'd be willing to make a more permissive
 * license. */

#include <sstream>

#include "errors.hpp"

namespace {

std::string
cl_error_msg(cl_int err)
{
	switch (err) {
	case CL_SUCCESS:
		return "success";
	case CL_DEVICE_NOT_FOUND:
		return "device not found";
	case CL_DEVICE_NOT_AVAILABLE:
		return "device not available";
	case CL_COMPILER_NOT_AVAILABLE:
		return "compiler not available";
	case CL_MEM_OBJECT_ALLOCATION_FAILURE:
		return "mem object allocation failure";
	case CL_OUT_OF_RESOURCES:
		return "out of resources";
	case CL_OUT_OF_HOST_MEMORY:
		return "out of host memory";
	case CL_PROFILING_INFO_NOT_AVAILABLE:
		return "profiling info not available";
	case CL_MEM_COPY_OVERLAP:
		return "mem copy overlap";
	case CL_IMAGE_FORMAT_MISMATCH:
		return "image format mismatch";
	case CL_IMAGE_FORMAT_NOT_SUPPORTED:
		return "image format not supported";
	case CL_BUILD_PROGRAM_FAILURE:
		return "build program failure";
	case CL_MAP_FAILURE:
		return "map failure";
	case CL_MISALIGNED_SUB_BUFFER_OFFSET:
		return "misaligned sub buffer offset";
	case CL_EXEC_STATUS_ERROR_FOR_EVENTS_IN_WAIT_LIST:
		return "exec status error for events in wait list";

	case CL_INVALID_VALUE:
		return "invalid value";
	case CL_INVALID_DEVICE_TYPE:
		return "invalid device type";
	case CL_INVALID_PLATFORM:
		return "invalid platform";
	case CL_INVALID_DEVICE:
		return "invalid device";
	case CL_INVALID_CONTEXT:
		return "invalid context";
	case CL_INVALID_QUEUE_PROPERTIES:
		return "invalid queue properties";
	case CL_INVALID_COMMAND_QUEUE:
		return "invalid command queue";
	case CL_INVALID_HOST_PTR:
		return "invalid host ptr";
	case CL_INVALID_MEM_OBJECT:
		return "invalid mem object";
	case CL_INVALID_IMAGE_FORMAT_DESCRIPTOR:
		return "invalid image format descriptor";
	case CL_INVALID_IMAGE_SIZE:
		return "invalid image size";
	case CL_INVALID_SAMPLER:
		return "invalid sampler";
	case CL_INVALID_BINARY:
		return "invalid binary";
	case CL_INVALID_BUILD_OPTIONS:
		return "invalid build options";
	case CL_INVALID_PROGRAM:
		return "invalid program";
	case CL_INVALID_PROGRAM_EXECUTABLE:
		return "invalid program executable";
	case CL_INVALID_KERNEL_NAME:
		return "invalid kernel name";
	case CL_INVALID_KERNEL_DEFINITION:
		return "invalid kernel definition";
	case CL_INVALID_KERNEL:
		return "invalid kernel";
	case CL_INVALID_ARG_INDEX:
		return "invalid arg index";
	case CL_INVALID_ARG_VALUE:
		return "invalid arg value";
	case CL_INVALID_ARG_SIZE:
		return "invalid arg size";
	case CL_INVALID_KERNEL_ARGS:
		return "invalid kernel args";
	case CL_INVALID_WORK_DIMENSION:
		return "invalid work dimension";
	case CL_INVALID_WORK_GROUP_SIZE:
		return "invalid work group size";
	case CL_INVALID_WORK_ITEM_SIZE:
		return "invalid work item size";
	case CL_INVALID_GLOBAL_OFFSET:
		return "invalid global offset";
	case CL_INVALID_EVENT_WAIT_LIST:
		return "invalid event wait list";
	case CL_INVALID_EVENT:
		return "invalid event";
	case CL_INVALID_OPERATION:
		return "invalid operation";
	case CL_INVALID_GL_OBJECT:
		return "invalid gl object";
	case CL_INVALID_BUFFER_SIZE:
		return "invalid buffer size";
	case CL_INVALID_MIP_LEVEL:
		return "invalid mip level";
	case CL_INVALID_GLOBAL_WORK_SIZE:
		return "invalid global work size";
#ifdef CL_INVALID_PROPERTY
	case CL_INVALID_PROPERTY:
		return "invalid property";
#endif

	case CL_PLATFORM_NOT_FOUND_KHR:
		return "platform not found";

	case CL_DEVICE_PARTITION_FAILED_EXT:
		return "device partition failed";
	case CL_INVALID_PARTITION_COUNT_EXT:
		return "invalid partition count";
	case CL_INVALID_PARTITION_NAME_EXT:
		return "invalid partition name";
	default:;
	}

	std::ostringstream out;
	out << "unknown (" << err << ')';
	return out.str();
}

} // end anon namespace

Opencl_error::Opencl_error(cl_int errval) :
	msg(cl_error_msg(errval)),
	errval(errval)
{
}

Opencl_error::Opencl_error(const std::string &msg, cl_int errval) :
	msg(msg),
	errval(errval)
{
	this->msg += ": ";
	this->msg += cl_error_msg(errval);
}
