/* Copyright 2010 Markus Peloquin <markus@cs.wisc.edu>,
 * Lena Olson <lena@cs.wisc.edu>
 *
 * You can't do anything with this but read and execute it privately.  If you
 * want to do more, ask us if we'd be willing to make a more permissive
 * license. */

#include <iostream>

#include <CL/cl.hpp>

#include "HRTimer.h"
#include "openclk.hpp"
#include "log.hpp"

extern char *__progname;

namespace {

class Fib : public Alg_spec<cl_uint> {
public:
	Fib(unsigned n)
	{
		add_arg(static_cast<cl_uint>(n));
	}

protected:
	virtual std::string name() const
	{	return "fib"; }

	virtual void reduce(const cl_uint *child_results, size_t len,
	    cl_uint *result)
	{
		cl_uint sum = 0;
		for (size_t i = 0; i < len; i++)
			sum += child_results[i];
		//std::cerr << child_results[0] << " + " << child_results[1];
		//std::cerr << " = " << sum << '\n';
		*result = sum;
	}

	virtual unsigned sub_tasks() const
	{	return 2; }
};

// wooo!
unsigned
fib(Openclk &openclk, unsigned n)
{
	Fib fib(n);
	return fib.run(openclk);
}

} // end anon namespace

int
main(int argc, char **argv)
{

	int num=10;
	// let's get size as an argument
	if (argc > 1){
	  num=atoi(argv[1]);
	}

	const char *source_files[] = {
		"fib.cl",
		//"kernels.cl",
	};
	size_t count = sizeof(source_files) / sizeof(*source_files);

	// initialize
	Openclk openclk(source_files, source_files + count);

	unsigned result;

	double start, end;
	init_timer();
	start = get_time_ns();

	result = fib(openclk, num);
	end = get_time_ns();
	std::cout << "fib result is " << result << '\n';
	std::cout << "Time: " << (end - start) / 1000000000.0 << " s\n"; 

	/*
	result = sum(openclk);
	std::cout << "sum is " << result << '\n';
	*/
	return 0;
}
