/* Copyright 2010 Markus Peloquin <markus@cs.wisc.edu>,
 * Lena Olson <lena@cs.wisc.edu>
 *
 * You can't do anything with this but read and execute it privately.  If you
 * want to do more, ask us if we'd be willing to make a more permissive
 * license. */

__kernel void knapsack(
    /* params */
    __constant int *weights, //read-only
    __constant int *values, //read-only
    __global int *best_so_far, //read-write

    unsigned max_tid,

    /* args */
    __constant int *arg_index,
    __constant int *arg_c,
    __constant int *arg_n,
    __constant int *arg_v,

    __global int *result,
    __global unsigned *spawn,

    /* spawn args */
    __global int *spawn_index,
    __global int *spawn_c,
    __global int *spawn_n,
    __global int *spawn_v
)
{
     unsigned tid = get_global_id(0);
     if (tid > max_tid) return;

     int index = arg_index[tid];
     int c = arg_c[tid];
     int n = arg_n[tid];
     int v = arg_v[tid];

     /* base case: full knapsack or no items */
     if (c < 0){
       result[tid] = INT_MIN;
       spawn[tid] = false;
       return;
     }

     if (n == 0 || c == 0){
       result[tid] = v;
       spawn[tid] = false;
       if (v > *best_so_far){
         *best_so_far = v;
       }
       return;             /* feasible solution, with value v */
     }

     int ub = v + c * values[index] / weights[index];
     if (ub < *best_so_far) {
       /* prune ! */
       result[tid] = INT_MIN;
       spawn[tid] = false;
       return;
     }

     int offset = tid * 2;
     spawn_index += offset;
     spawn_c += offset;
     spawn_n += offset;
     spawn_v += offset;

     /* 
      * compute the best solution without the current item in the knapsack 
      */
     //without = knapsack(weights, values, index+1, c, n - 1, v);
     spawn_index[0] = index + 1;
     spawn_c[0] = c;
     spawn_n[0] = n - 1;
     spawn_v[0] = v;

     /* compute the best solution with the current item in the knapsack */
     //with = knapsack(weights, values, index+1, c - weights[index], n - 1, v + values[index]);
     spawn_index[1] = spawn_index[0];
     spawn_c[1] = c - weights[index];
     spawn_n[1] = spawn_n[0];
     spawn_v[1] = v + values[index];

     spawn[tid] = true;
}

/* vim: set filetype=c : */
