/* Copyright 2010 Markus Peloquin <markus@cs.wisc.edu>,
 * Lena Olson <lena@cs.wisc.edu>
 *
 * You can't do anything with this but read and execute it privately.  If you
 * want to do more, ask us if we'd be willing to make a more permissive
 * license. */

#ifndef OPENCLKLOG_H
#define OPENCLKLOG_H

#include <algorithm>
#include <iostream>
#include <iomanip>
#include <vector>

#include "HRTimer.h"

extern std::vector<const char *> cpu_events;
extern std::vector<double> cpu_times;
extern std::vector<const char *> gpu_events;
extern std::vector<double> gpu_times;

inline
void logEvent(const char *event){
#ifdef TIMING
	cpu_times.push_back(get_time_ns());
	cpu_events.push_back(event);
#endif
}

inline
void logGPUEvent(const char *event){
#ifdef TIMING
	gpu_times.push_back(get_time_ns());
	gpu_events.push_back(event);
#endif
}

struct Event_pair_cmp {
	typedef std::pair<const char *, double> pair;
	bool operator()(const pair &a, const pair &b) const
	{
		return a.second < b.second;
	}
};

inline
void printLog(){
#ifdef TIMING
	std::vector<std::pair<const char *, double> > events;
	for (size_t i=0; i < cpu_events.size(); i++)
		events.push_back(std::make_pair(cpu_events[i], cpu_times[i]- cpu_times[0]));
	for (size_t i=0; i < gpu_events.size(); i++)
		events.push_back(std::make_pair(gpu_events[i], gpu_times[i]-cpu_times[0]));
	std::sort(events.begin(), events.end(), Event_pair_cmp());


	std::cout << std::setprecision(8);// << std::fixed;
	for (size_t i=0; i < events.size(); i++) {
		std::cout << "EVENT " << events[i].first << ": "
		    << events[i].second << "\n";
	}
#endif
}

#endif
