/* Copyright 2010 Markus Peloquin <markus@cs.wisc.edu>,
 * Lena Olson <lena@cs.wisc.edu>
 *
 * You can't do anything with this but read and execute it privately.  If you
 * want to do more, ask us if we'd be willing to make a more permissive
 * license. */

#define MAX_COLS 4

void __kernel matmul(
	/* params */
	__constant float *A, /*read only*/
	__constant float *B, /*read only*/
	__global float *C, /*write only*/
	unsigned m,
	unsigned n,
	unsigned o,

	unsigned max_tid,

	/* args */
	__constant unsigned *arg_c0,
	__constant unsigned *arg_c1,

	/* results  */
	__global unsigned *result,
	__global unsigned *spawn,

	/* spawn args */
	__global unsigned *spawn_c0,
	__global unsigned *spawn_c1
)
{
	unsigned tid = get_global_id(0);
	if (tid > max_tid) return;
	unsigned c0 = arg_c0[tid];
	unsigned c1 = arg_c1[tid];

	if (c1 - c0 > MAX_COLS) {
		unsigned avg = (c0 + c1) / 2;
		spawn_c0 += tid*2;
		spawn_c1 += tid*2;

		spawn_c0[0] = c0;
		spawn_c1[0] = avg;

		spawn_c0[1] = avg;
		spawn_c1[1] = c1;
		spawn[tid] = true;
		return;
	}

	// the first two loops can be reordered, but it's a little faster like
	// this on the CPU
	for (unsigned col = c0; col < c1; col++){
		for (unsigned row = 0; row < m; row++) {
			unsigned C_idx = row + col * m;
			C[C_idx] = 0;
			for (unsigned k = 0; k < n; k++){
				C[C_idx] += A[row + k * m] * B[k + col * n];
			}
		}
	}
	spawn[tid] = false;
}

/* vim: set filetype=c : */
