/* Copyright 2010 Markus Peloquin <markus@cs.wisc.edu>,
 * Lena Olson <lena@cs.wisc.edu>
 *
 * You can't do anything with this but read and execute it privately.  If you
 * want to do more, ask us if we'd be willing to make a more permissive
 * license. */

#include <cstdio>
#include <cstdlib>
#include <iostream>

#include "HRTimer.h"
#include "openclk.hpp"

namespace {
  class Matmul : public Alg_spec<cl_uint> {
    public:
      Matmul(cl_float *A, cl_float *B, cl_float *C, cl_uint m, cl_uint n,
	     cl_uint o)
      {
        add_param(A, m*n*sizeof(cl_float), Argument_buffer::read);
        add_param(B, n*o*sizeof(cl_float), Argument_buffer::read);
        add_param(C, m*o*sizeof(cl_float), Argument_buffer::write);
        add_param(m);
        add_param(n);
        add_param(o);
        add_arg(static_cast<cl_uint>(0));
	add_arg(o);
      }

    protected:
      virtual std::string name() const
      { return "matmul"; }

      virtual void reduce(const cl_uint *child_results, size_t len,
                          cl_uint *result)
      {
      }

      virtual unsigned sub_tasks() const
      { return 2; }
  };
  
  // wooo!
  void matmul(Openclk &openclk, cl_float *A, cl_float *B, cl_float *C,
    cl_uint m, cl_uint n, cl_uint o)
  {
    Matmul matmul(A, B, C, m, n, o);
    matmul.run(openclk);
  }

} // end anon namespace

void
fill_random(cl_float *arr, size_t num)
{
	while (num--)
		*arr++ = rand() % 100;
}

int
main(int argc, char **argv)
{
#ifdef CHECK
	cl_uint m = 4, n = 4, o = 4;
	cl_float A[] = {1, 3, -2, 2,   0, 0, 1, 0,     1, 1, 0, 1,  2, -1, 1, 1 };
	cl_float B[] = {-2, -3, 5, -1, -1, -1, 3, -1,  0, 1, 0, 0,  3, 4, -7, 2 };
	cl_float C[16];
#else
	// dimensions
	cl_uint m = 10;
	cl_uint n = 10;
	cl_uint o = 10;

	if (argc > 1) {
		int dimen = atoi(argv[1]);
		m = n = o = dimen;
		std::cout << "dimensions: " << dimen << '\n';
	}

	srand(0);
	cl_float *A = new cl_float[m * n];
	cl_float *B = new cl_float[n * o];
	cl_float *C = new cl_float[m * o];

	fill_random(A, m * n);
	fill_random(B, n * o);
#endif

	const char *source_files[] = {
		"matmul.cl",
	};
	size_t count = sizeof(source_files) / sizeof(*source_files);

	// initialize 
	Openclk openclk(source_files, source_files + count);

	init_timer();
	double begin = get_time_ns();
	matmul(openclk, A, B, C, m, n, o);
	double end = get_time_ns();

#ifdef CHECK
	std::cout << "result (should be identity):\n";
	for (size_t i = 0; i < m * o; i++)
		std::cout << C[i] << ' ';
	std::cout << '\n';
#endif

	std::cout << "time: " << (end - begin) / 1000000000.0 << " s\n";

#ifndef CHECK
	delete[] A;
	delete[] B;
	delete[] C;
#endif

	return 0;
}
