/* Copyright 2010 Markus Peloquin <markus@cs.wisc.edu>,
 * Lena Olson <lena@cs.wisc.edu>
 *
 * You can't do anything with this but read and execute it privately.  If you
 * want to do more, ask us if we'd be willing to make a more permissive
 * license. */

#include <cstdio>
#include <cstdlib>
#include <iostream>

#include "HRTimer.h"

//#define CHECK

#ifdef CHECK
const unsigned MAX_COLS = 1;
#else
const unsigned MAX_COLS = 4;
#endif

void
matmul(const float *A, const float *B, float *C,
    unsigned m, unsigned n, unsigned o,
    unsigned c0, unsigned c1)
{
	if (c1 - c0 > MAX_COLS) {
		unsigned avg = (c0 + c1) / 2;
		matmul(A, B, C, m, n, o, c0, avg);
		matmul(A, B, C, m, n, o, avg, c1);
		return;
	}

	// the first two loops can be reordered, but it's a little faster like
	// this on the CPU
	for (unsigned col = c0; col < c1; col++) 
		for (unsigned row = 0; row < m; row++) {
			unsigned C_idx = row + col * m;
			C[C_idx] = 0;
			for (unsigned k = 0; k < n; k++)
				C[C_idx] += A[row + k * m] * B[k + col * n];
		}
}

void
matmul(const float *A, const float *B, float *C,
    unsigned m, unsigned n, unsigned o)
{
	matmul(A, B, C, m, n, o, 0, o);
}

void
fill_random(float *arr, size_t num)
{
	while (num--)
		*arr++ = rand() % 100;
}

int
main(int argc, char **argv)
{
#ifdef CHECK
	unsigned m = 4, n = 4, o = 4;
	float A[] = {1, 3, -2, 2,   0, 0, 1, 0,     1, 1, 0, 1,  2, -1, 1, 1 };
	float B[] = {-2, -3, 5, -1, -1, -1, 3, -1,  0, 1, 0, 0,  3, 4, -7, 2 };
	float C[16];
#else
	// dimensions
	unsigned m = 10;
	unsigned n = 10;
	unsigned o = 10;

	srand(0);
	float *A = new float[m * n];
	float *B = new float[n * o];
	float *C = new float[m * o];

	fill_random(A, m * n);
	fill_random(B, n * o);
#endif

	init_timer();
	double begin = get_time_ns();
	matmul(A, B, C, m, n, o);
	double end = get_time_ns();

#ifdef CHECK
	std::cout << "result (should be identity):\n";
	for (size_t i = 0; i < m * o; i++)
		std::cout << C[i] << ' ';
	std::cout << '\n';
#endif

	std::cout << "time: " << (end - begin) / 1000000000.0 << " s\n";

#ifndef CHECK
	delete[] A;
	delete[] B;
	delete[] C;
#endif

	return 0;
}
