/* Copyright 2010 Markus Peloquin <markus@cs.wisc.edu>,
 * Lena Olson <lena@cs.wisc.edu>
 *
 * You can't do anything with this but read and execute it privately.  If you
 * want to do more, ask us if we'd be willing to make a more permissive
 * license. */

#include <cstring>
#include <math.h>
#include <iostream>

#include <CL/cl.hpp>

#include "HRTimer.h"
#include "openclk.hpp"

extern char *__progname;

namespace {

  class Queens : public Alg_spec<cl_uint> {
    public:
      Queens(cl_uint* queen_array, int num_queens, int dim)
      {
        dimensions = dim;
        add_arg(queen_array, dimensions * sizeof(cl_uint));
        add_arg(static_cast<cl_uint>(num_queens));
        add_param(static_cast<cl_uint>(dimensions));
      }

    protected:
      virtual std::string name() const
      { return "queens"; }

      virtual void reduce(const cl_uint *child_results, size_t len,
                          cl_uint *result)
      {
        cl_uint sum = 0;
        for (size_t i = 0; i < len; i++)
          sum += child_results[i];
        *result = sum;
      }

      virtual unsigned sub_tasks() const
      { return dimensions; }

      int dimensions;
  };

// wooo!
  unsigned
     queens(Openclk &openclk, cl_uint * queen_array, int num_queens, int dimensions)
  {
    Queens queens(queen_array, num_queens, dimensions);
    return queens.run(openclk);
  }

} // end anon namespace



void display_solution(char * queens, int dimensions){
  for (int i=0; i < dimensions; i++){
    for (int j=0; j < dimensions; j++){
      if (queens[i] == j ) std::cout << "X";
      else std::cout << "_";
    }
    std::cout << "\n";
  }
}

int main(int argc, char** argv){
  int num=8;
  // let's get size as an argument
  if (argc > 1){
    num=atoi(argv[1]);
  }

  const char *source_files[] = {
    "queens.cl",
  };
  size_t count = sizeof(source_files) / sizeof(*source_files);

  // initialize 
  Openclk openclk(source_files, source_files + count);

  unsigned result;
  cl_uint *queens_array = new cl_uint[num];

  double start, end;
  init_timer();
  start = get_time_ns();

   result = queens(openclk, queens_array, 0, num);

  end = get_time_ns();
  std::cout << "queens result is " << result << '\n';
  std::cout << "Time: " << (end - start) / 1000000000.0 << " s\n";  
  return 0;
}
