/* Copyright 2010 Markus Peloquin <markus@cs.wisc.edu>,
 * Lena Olson <lena@cs.wisc.edu>
 *
 * You can't do anything with this but read and execute it privately.  If you
 * want to do more, ask us if we'd be willing to make a more permissive
 * license. */

#include <cstdlib>
#include <cstring>
#include <iostream>

#include "HRTimer.h"

void display_solution(char * queens, int dimensions){
  for (int i=0; i < dimensions; i++){
    for (int j=0; j < dimensions; j++){
      if (queens[i] == j ) std::cout << "X";
      else std::cout << "_";
    }
    std::cout << "\n";
  }
}

bool isConsistent(char * queens, int fill, int new_queen){
  for (int i =0; i < fill; i++){
    if (queens[i] == new_queen){
      //whoops, same column
      return false;
    }
    if (abs(queens[i] - new_queen) == fill - i){
      //whoops, same diagonal
      return false;
    }
  }
  return true;
}

// REASONABLE CPU CODE
unsigned count_solutions(char * queens, int num_queens, int dimensions){
  if (num_queens >= dimensions){
//     std::cout << "found solution: \n";
//     display_solution(queens, dimensions);
    //found a solution
    delete[] queens;
    return 1;
  }
  unsigned count=0;
  for (int i = 0; i < dimensions; i++){ //iterate through columns
    if (isConsistent(queens, num_queens, i)){
      char *new_queens = new char[dimensions];
      memcpy(new_queens, queens, dimensions);
      new_queens[num_queens] = i;
      count += count_solutions(new_queens, num_queens+1, dimensions);
    }
  }

  delete[] queens;
  return count;
}


int main(int argc, char** argv){
  int num=8;
  // let's get size as an argument
  if (argc > 1){
    num=atoi(argv[1]);
  }


  unsigned result;
  char *queens = new char[num];

  double start, end;
  init_timer();
  start = get_time_ns();

  result = count_solutions(queens, 0, num);
  end = get_time_ns();
  std::cout << "queens result is " << result << '\n';
  std::cout << "Time: " << (end - start) / 1000000000.0 << " s\n"; 


  return 0;
}
