/* Copyright 2010 Markus Peloquin <markus@cs.wisc.edu>,
 * Lena Olson <lena@cs.wisc.edu>
 *
 * You can't do anything with this but read and execute it privately.  If you
 * want to do more, ask us if we'd be willing to make a more permissive
 * license. */

/* wikipedia's pseudocode for in-place quicksort
 function quicksort(array, left, right)
     var pivot, leftIdx = left, rightIdx = right
     if right - left > 0
         pivot = (left + right) / 2
         while leftIdx <= pivot and rightIdx >= pivot
             while array[leftIdx] < array[pivot] and leftIdx <= pivot
                 leftIdx = leftIdx + 1
             while array[rightIdx] > array[pivot] and rightIdx >= pivot
                 rightIdx = rightIdx - 1;
             swap array[leftIdx] with array[rightIdx]
             leftIdx = leftIdx + 1
             rightIdx = rightIdx - 1
             if leftIdx - 1 == pivot
                 pivot = rightIdx = rightIdx + 1
             else if rightIdx + 1 == pivot
                 pivot = leftIdx = leftIdx - 1
         quicksort(array, left, pivot - 1)
         quicksort(array, pivot + 1, right)
*/

__kernel void quacksort(
			__global unsigned *array, //readwritev;

			unsigned max_tid,

                        __constant int *left,
                        __constant int *right,

                        __global unsigned *result, //placeholder
                        __global unsigned *spawn,

                        __global unsigned *spawn_left,
                        __global unsigned *spawn_right)
{
  unsigned tid = get_global_id(0);
  if (tid > max_tid) return;
  int leftIdx = left[tid];
  int rightIdx = right[tid];
  unsigned pivot = 0;
  unsigned tmp = 0;
  if (right[tid] > left[tid]){
    pivot = (left[tid] + right[tid]) / 2;
    while (leftIdx <= pivot && rightIdx >= pivot){
      while (array[leftIdx] < array[pivot] && leftIdx <= pivot){
        leftIdx++;
      }
      while (array[rightIdx] > array[pivot] && rightIdx >= pivot){
        rightIdx--;
      }
      tmp = array[leftIdx];
      array[leftIdx] = array[rightIdx];
      array[rightIdx] = tmp;

      leftIdx++;
      rightIdx--;
      if (leftIdx - 1 == pivot){
        pivot = ++rightIdx;
      }
      else if (rightIdx + 1 == pivot){
        pivot = --leftIdx;
      }
    }
    //spawn quacksort(array, left, pivot -1)
    spawn_left[tid*2] = left[tid];
    spawn_right[tid*2] = pivot - 1;
    //spawn quacksort(array, pivot+1, right)
    spawn_left[tid*2 + 1] = pivot+1;
    spawn_right[tid*2 + 1] = right[tid];
    spawn[tid] = true; 
  }
  else {
    spawn[tid] = false;
  }
}

/* vim: set filetype=c : */
