/* Copyright 2010 Markus Peloquin <markus@cs.wisc.edu>,
 * Lena Olson <lena@cs.wisc.edu>
 *
 * You can't do anything with this but read and execute it privately.  If you
 * want to do more, ask us if we'd be willing to make a more permissive
 * license. */

#include <cstring>
#include <cstdlib>
#include <math.h>
#include <iostream>

#include <CL/cl.hpp>

#include "HRTimer.h"
#include "openclk.hpp"

extern char *__progname;

namespace {

  class Quacksort : public Alg_spec<cl_uint> {
    public:
      Quacksort(cl_uint* array, cl_int len)
      {
        add_param(array, len * sizeof(cl_uint), Argument_buffer::read_write);
        add_arg(static_cast<cl_int>(0));
        add_arg(len - 1);
      }

    protected:
      virtual std::string name() const
      { return "quacksort"; }

      virtual void reduce(const cl_uint *child_results, size_t len,
                          cl_uint *result)
      {
	return;
      }

      virtual unsigned sub_tasks() const
      { return 2; }

  };

// wooo!
  unsigned
     quacksort(Openclk &openclk, cl_uint * array, cl_uint len)
  {
    Quacksort quacksort(array, len);
    return quacksort.run(openclk);
  }

} // end anon namespace


int main(int argc, char** argv){
  cl_int length=10; //list length
  srand(0);
  if (argc > 1){
    length=atoi(argv[1]);
  }
  if (argc > 2){
    srand(atoi(argv[2]));
  }
  
  cl_uint * array = new cl_uint[length];
  
  for (cl_int i=0; i < length; i++){
    array[i] = rand();
  }

  const char *source_files[] = {
    "quacksort.cl",
  };
  size_t count = sizeof(source_files) / sizeof(*source_files);

  // initialize 
  Openclk openclk(source_files, source_files + count);

  
  
  double start, end;
  init_timer();
  start = get_time_ns();

  quacksort(openclk, array, length);

  end = get_time_ns();
  bool failed = false;
  std::cout << array[0] << " ";
  for (cl_int i=1; i < length; i++){
    std::cout << array[i] << " ";
    if (array[i] < array[i-1]){
      failed = true;
    }
  }
  delete[] array;

//   std::cout << "\n";
  if (failed){
    std::cout << "Sort failed!!!\n";
  }
  else {
    std::cout << "Sort succeeded.\n";
  }
  std::cout << "Sorted " << length << " elements\n";
  std::cout << "Time: " << (end - start) / 1000000000.0 << " s\n";
  return 0;
}
