/* Copyright 2010 Markus Peloquin <markus@cs.wisc.edu>,
 * Lena Olson <lena@cs.wisc.edu>
 *
 * You can't do anything with this but read and execute it privately.  If you
 * want to do more, ask us if we'd be willing to make a more permissive
 * license. */

#include <cstring>
#include <cstdlib>
#include <math.h>
#include <iostream>

#include "HRTimer.h"

extern char *__progname;

void quacksort(unsigned *array, int left, int right)
{
  int leftIdx = left;
  int rightIdx = right;
  unsigned pivot = 0;
  unsigned tmp = 0;
  if (right > left){
    pivot = (left + right) / 2;
    while (leftIdx <= pivot && rightIdx >= pivot){
      while (array[leftIdx] < array[pivot] && leftIdx <= pivot){
        leftIdx++;
      }
      while (array[rightIdx] > array[pivot] && rightIdx >= pivot){
        rightIdx--;
      }
      tmp = array[leftIdx];
      array[leftIdx] = array[rightIdx];
      array[rightIdx] = tmp;

      leftIdx++;
      rightIdx--;
      if (leftIdx - 1 == pivot){
        pivot = ++rightIdx;
      }
      else if (rightIdx + 1 == pivot){
        pivot = --leftIdx;
      }
    }
    quacksort(array, left, pivot -1);

    quacksort(array, pivot+1, right);

  }
}

int main(int argc, char** argv){
  unsigned length=1000; //list length
  srand(0);
  if (argc > 1){
    length=atoi(argv[1]);
  }
  if (argc > 2){
    srand(atoi(argv[2]));
  }
  
  unsigned * array = new unsigned[length];
  
  for (unsigned i=0; i < length; i++){
    array[i] = rand();
  }
  
//   for (unsigned i=0; i < length; i++){
//     std::cout << array[i] << " ";
//   }
//   std::cout << "\n";
  
  double start, end;
  init_timer();
  start = get_time_ns();

  quacksort(array, 0, length-1);

  end = get_time_ns();
  bool failed = false;
//   std::cout << array[0] << " ";
  for (unsigned i=1; i < length; i++){
//     std::cout << array[i] << " ";
    if (array[i] < array[i-1]){
      failed = true;
    }
  }
//   std::cout << "\n";
  if (failed){
    std::cout << "Sort failed!!!\n";
  }
  else {
    std::cout << "Sort succeeded.\n";
  }

  std::cout << "Sorted " << length << " elements\n";
  std::cout << "Time: " << (end - start) / 1000000000.0 << " s\n";
  return 0;
}
