/* Copyright 2010 Markus Peloquin <markus@cs.wisc.edu>,
 * Lena Olson <lena@cs.wisc.edu>
 *
 * You can't do anything with this but read and execute it privately.  If you
 * want to do more, ask us if we'd be willing to make a more permissive
 * license. */

bool isConsistent(__constant int * queens, int fill, int new_queen){
  for (int i =0; i < fill; i++){
    if (queens[i] == new_queen){
      //whoops, same column
      return false;
    }
    if (abs(queens[i] - new_queen) == fill - i){
      //whoops, same diagonal
      return false;
    }
  }
  return true;
}

void copyCharBuffer(__global int * destination, __constant int * source, int length){
  for (int i=0; i < length; i++){
    destination[i] = source[i];
  }
}   

__kernel void
queens(
    /* params */
    unsigned dimensions,

    unsigned max_tid,

    /* args */
    __constant int *queens_array,
    __constant unsigned *num_queens,

    __global unsigned *result,
    __global unsigned *spawn,

    /* spawn args */
    __global int *spawn_array,
    __global unsigned *spawn_num)
{
  unsigned tid = get_global_id(0);
  if (tid > max_tid) return;
  __constant int * queens = queens_array + tid * dimensions;
  unsigned num = num_queens[tid];
  result[tid] = 0;

  if (num && !isConsistent(queens, num-1, queens[num-1])){
    spawn[tid] = false;
    result[tid] = 0;
    return;
  }
  if (num >= dimensions){
//      std::cout << "found solution: \n";
//      display_solution(queens, dimensions);
    //found a solution
    result[tid] = 1;
    spawn[tid] = false;
    return;
  }

  for (int i = 0; i < dimensions; i++){ //iterate through columns
    __global int * new_queens = spawn_array + (tid*dimensions + i)*dimensions;
    copyCharBuffer(new_queens, queens, dimensions);
    new_queens[num] = i;
    spawn_num[tid*dimensions + i] = num+1;
  }

  spawn[tid] = true;

  /* sync (always at end) */
}

/* vim: set filetype=c : */
