/* Copyright 2010 Markus Peloquin <markus@cs.wisc.edu>,
 * Lena Olson <lena@cs.wisc.edu>
 *
 * You can't do anything with this but read and execute it privately.  If you
 * want to do more, ask us if we'd be willing to make a more permissive
 * license. */

#ifndef OPENCLK_QUEUE_HPP
#define OPENCLK_QUEUE_HPP

#include <queue>

#include <pthread.h>

// uses coarse-grained synchronization
template <typename T>
class Queue {
public:
	typedef T value_type;

	Queue() :
		_queue()
	{
		pthread_mutex_init(&_lock, 0);
		//pthread_cond_init(&_empty, 0);
		pthread_cond_init(&_not_empty, 0);
	}
	~Queue()
	{
		pthread_cond_destroy(&_not_empty);
		//pthread_cond_destroy(&_empty);
		pthread_mutex_destroy(&_lock);
	}

	value_type pop()
	{
		pthread_mutex_lock(&_lock);
		while (_queue.empty())
			pthread_cond_wait(&_not_empty, &_lock);
		value_type resp = _queue.front();
		_queue.pop();
		if (!_queue.empty())
			pthread_cond_signal(&_not_empty);
		pthread_mutex_unlock(&_lock);
		return resp;
	}
	void push(const value_type &t)
	{
		pthread_mutex_lock(&_lock);
		_queue.push(t);
		pthread_cond_signal(&_not_empty);
		pthread_mutex_unlock(&_lock);
	}

private:
	std::queue<value_type>	_queue;
	pthread_mutex_t		_lock;
	//pthread_cond_t		_empty;
	pthread_cond_t		_not_empty;
};

#endif
